local a = {}
a.__index = a

function a:new(name)
	self = setmetatable({}, self)
	self.source  = love.audio.newSource(name, 'stream')
	
	local bytespersample    = self.source:getChannelCount() * 2 -- 16-bit sound
	local max_fft_frequency = 2048 * 8
	
	self.decoder = love.sound.newDecoder(name, max_fft_frequency * bytespersample)
	
	local meta = getmetatable(self.source)
	for k, v in pairs(meta.__index) do
		self[k] = function(self, ...) return self.source[k](self.source, ...) end
	end
	
	self.source:setLooping(true)
	return self
end

function a:getBytes(dt)
	local l = self.source:tell()
	l = l - dt
	l = l < 0 and 0 or l
	self.decoder:seek(l)
	return self.decoder:decode()
end

local o = a:new('tojk.mp3')
o:play()

love.graphics.setLineStyle('rough')
love.graphics.setLineJoin( 'none' )

local v = 1
function love.wheelmoved(mx, my)
	v = v * (my > 0 and 2 or 0.5)
	o:setVolume(v)
end

function love.filedropped(f)
	local n = f:getFilename():lower()
	if n:find("%.mp3$") or n:find("%.ogg$") or n:find("%.wav$") or n:find("%.flac$") then
		o:stop()
		o = a:new(f)
		o:setVolume(v)
		o:play()
	end
end

function love.draw()
	

	love.window.setTitle(love.timer.getFPS())
	love.graphics.setColor(1, 1, 1)
	love.graphics.print('Drag and drop an mp3/ogg/wav/flac file')
	love.graphics.print('Vol: ' .. v, 0, 30)

	local graph = {}
	local data = o:getBytes(0 or love.timer.getDelta())
	if not data then return end
	
	local w, h = love.graphics.getDimensions()
	local gp = h / 2
	local gh = h / 2
	
	local n = -1

	local left  = {}
	local right = {}
	local fft   = {}
	
	local len = data:getSampleCount() / 2
	local fft = {}
	
	local step = w / len * 16
	local f1, p1 = 0, 0
	for i = 1, len do
		local s1, s2 = data:getSample(i, 1), data:getSample(i, 2)
		if i % 8 == 0 then
			f1 = f1 + 1
			fft[f1] = (s1 + s2) * .5
		end
		if i % 16 == 0 then
			p1 = p1 + 1
			local m = p1 * 2
			left[m - 1]  = p1 * step
			left[m]  = data:getSample((p1 - 1) * 2, 1) * gh + gp
			right[m - 1] = p1 * step
			right[m] = data:getSample((p1 - 1) * 2, 2) * gh + gp
		end
	end
	
	love.graphics.setColor(.2, .2, 1, .5)
	love.graphics.line(left)
	love.graphics.setColor(.2, 1, .2, .5)
	love.graphics.line(right)
		
	fft = require'fft'(fft)
	
	local _h = #fft / 2
	local graph = {}
	
	-- we need only half of fft
	local len  = #fft
	local step = w / len * 2
	for i = 1, #fft / 2 do
		local p = fft[i]
		local mod = (p.i^2 + p.r^2) ^ .5 * (i / len)
		graph[(i * 2) - 1] = i * step
		graph[(i * 2)] = -mod * 50 + h - 5
	end
	
	love.graphics.setColor(1, 1, 1)
	love.graphics.line(graph)

	love.graphics.print('Do not use this music in commercial stuff pls', 300)
end